package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLLightParameter implements GLEnum {
	AMBIENT(0x1200),
	DIFFUSE(0x1201),
	SPECULAR(0x1202),
	POSITION(0x1203),
	SPOT_DIRECTION(0x1204),
	SPOT_EXPONENT(0x1205),
	SPOT_CUTOFF(0x1206),
	CONSTANT_ATTENUATION(0x1207),
	LINEAR_ATTENUATION(0x1208),
	QUADRATIC_ATTENUATION(0x1209);
	
	private final int glEnum;

	GLLightParameter(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLLightParameter> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLLightParameter fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1200 -> AMBIENT;
			case 0x1201 -> DIFFUSE;
			case 0x1202 -> SPECULAR;
			case 0x1203 -> POSITION;
			case 0x1204 -> SPOT_DIRECTION;
			case 0x1205 -> SPOT_EXPONENT;
			case 0x1206 -> SPOT_CUTOFF;
			case 0x1207 -> CONSTANT_ATTENUATION;
			case 0x1208 -> LINEAR_ATTENUATION;
			case 0x1209 -> QUADRATIC_ATTENUATION;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLLightParameter");
		};
	}
}