package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLMapBufferAccessMask implements GLEnum {
	MAP_READ_BIT(0x1),
	MAP_WRITE_BIT(0x2),
	MAP_INVALIDATE_RANGE_BIT(0x4),
	MAP_INVALIDATE_BUFFER_BIT(0x8),
	MAP_FLUSH_EXPLICIT_BIT(0x10),
	MAP_UNSYNCHRONIZED_BIT(0x20),
	MAP_PERSISTENT_BIT(0x40),
	MAP_COHERENT_BIT(0x80);
	
	private final int glEnum;

	GLMapBufferAccessMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLMapBufferAccessMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLMapBufferAccessMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> MAP_READ_BIT;
			case 0x2 -> MAP_WRITE_BIT;
			case 0x4 -> MAP_INVALIDATE_RANGE_BIT;
			case 0x8 -> MAP_INVALIDATE_BUFFER_BIT;
			case 0x10 -> MAP_FLUSH_EXPLICIT_BIT;
			case 0x20 -> MAP_UNSYNCHRONIZED_BIT;
			case 0x40 -> MAP_PERSISTENT_BIT;
			case 0x80 -> MAP_COHERENT_BIT;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLMapBufferAccessMask");
		};
	}
}