package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLObjectIdentifier implements GLEnum {
	TEXTURE(0x1702),
	VERTEX_ARRAY(0x8074),
	BUFFER(0x82e0),
	SHADER(0x82e1),
	PROGRAM(0x82e2),
	QUERY(0x82e3),
	PROGRAM_PIPELINE(0x82e4),
	SAMPLER(0x82e6),
	FRAMEBUFFER(0x8d40),
	RENDERBUFFER(0x8d41),
	TRANSFORM_FEEDBACK(0x8e22);
	
	private final int glEnum;

	GLObjectIdentifier(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLObjectIdentifier> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLObjectIdentifier fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1702 -> TEXTURE;
			case 0x8074 -> VERTEX_ARRAY;
			case 0x82e0 -> BUFFER;
			case 0x82e1 -> SHADER;
			case 0x82e2 -> PROGRAM;
			case 0x82e3 -> QUERY;
			case 0x82e4 -> PROGRAM_PIPELINE;
			case 0x82e6 -> SAMPLER;
			case 0x8d40 -> FRAMEBUFFER;
			case 0x8d41 -> RENDERBUFFER;
			case 0x8e22 -> TRANSFORM_FEEDBACK;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLObjectIdentifier");
		};
	}
}