package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPathMetricMask implements GLEnum {
	GLYPH_WIDTH_BIT_NV(0x1),
	GLYPH_HEIGHT_BIT_NV(0x2),
	GLYPH_HORIZONTAL_BEARING_X_BIT_NV(0x4),
	GLYPH_HORIZONTAL_BEARING_Y_BIT_NV(0x8),
	GLYPH_HORIZONTAL_BEARING_ADVANCE_BIT_NV(0x10),
	GLYPH_VERTICAL_BEARING_X_BIT_NV(0x20),
	GLYPH_VERTICAL_BEARING_Y_BIT_NV(0x40),
	GLYPH_VERTICAL_BEARING_ADVANCE_BIT_NV(0x80),
	GLYPH_HAS_KERNING_BIT_NV(0x100),
	FONT_X_MIN_BOUNDS_BIT_NV(0x10000),
	FONT_Y_MIN_BOUNDS_BIT_NV(0x20000),
	FONT_X_MAX_BOUNDS_BIT_NV(0x40000),
	FONT_Y_MAX_BOUNDS_BIT_NV(0x80000),
	FONT_UNITS_PER_EM_BIT_NV(0x100000),
	FONT_ASCENDER_BIT_NV(0x200000),
	FONT_DESCENDER_BIT_NV(0x400000),
	FONT_HEIGHT_BIT_NV(0x800000),
	FONT_MAX_ADVANCE_WIDTH_BIT_NV(0x1000000),
	FONT_MAX_ADVANCE_HEIGHT_BIT_NV(0x2000000),
	FONT_UNDERLINE_POSITION_BIT_NV(0x4000000),
	FONT_UNDERLINE_THICKNESS_BIT_NV(0x8000000),
	FONT_HAS_KERNING_BIT_NV(0x10000000),
	FONT_NUM_GLYPH_INDICES_BIT_NV(0x20000000);
	
	private final int glEnum;

	GLPathMetricMask(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPathMetricMask> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPathMetricMask fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x1 -> GLYPH_WIDTH_BIT_NV;
			case 0x2 -> GLYPH_HEIGHT_BIT_NV;
			case 0x4 -> GLYPH_HORIZONTAL_BEARING_X_BIT_NV;
			case 0x8 -> GLYPH_HORIZONTAL_BEARING_Y_BIT_NV;
			case 0x10 -> GLYPH_HORIZONTAL_BEARING_ADVANCE_BIT_NV;
			case 0x20 -> GLYPH_VERTICAL_BEARING_X_BIT_NV;
			case 0x40 -> GLYPH_VERTICAL_BEARING_Y_BIT_NV;
			case 0x80 -> GLYPH_VERTICAL_BEARING_ADVANCE_BIT_NV;
			case 0x100 -> GLYPH_HAS_KERNING_BIT_NV;
			case 0x10000 -> FONT_X_MIN_BOUNDS_BIT_NV;
			case 0x20000 -> FONT_Y_MIN_BOUNDS_BIT_NV;
			case 0x40000 -> FONT_X_MAX_BOUNDS_BIT_NV;
			case 0x80000 -> FONT_Y_MAX_BOUNDS_BIT_NV;
			case 0x100000 -> FONT_UNITS_PER_EM_BIT_NV;
			case 0x200000 -> FONT_ASCENDER_BIT_NV;
			case 0x400000 -> FONT_DESCENDER_BIT_NV;
			case 0x800000 -> FONT_HEIGHT_BIT_NV;
			case 0x1000000 -> FONT_MAX_ADVANCE_WIDTH_BIT_NV;
			case 0x2000000 -> FONT_MAX_ADVANCE_HEIGHT_BIT_NV;
			case 0x4000000 -> FONT_UNDERLINE_POSITION_BIT_NV;
			case 0x8000000 -> FONT_UNDERLINE_THICKNESS_BIT_NV;
			case 0x10000000 -> FONT_HAS_KERNING_BIT_NV;
			case 0x20000000 -> FONT_NUM_GLYPH_INDICES_BIT_NV;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPathMetricMask");
		};
	}
}